[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Exchange DataFeed Data Model client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.ExchangeDataFeedDataModel/0.20.13)](https://pypi.org/project/fds.sdk.ExchangeDataFeedDataModel/v/0.20.13)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet’s Exchange DataFeed products provide access to consolidated real-time and delayed global exchange data. Proprietary technology normalizes over 250 global venues, 18+ million instruments, and 150+ data fields. Asset types integrated include equities, futures, options, warrants, fixed income, mutual funds, ETFs, indices, commodities, and FX rates. Innovative technology ensures reliability and provides scalability that allows clients to make requests based on a symbol list or an exchange. Reduce development time by powering proprietary and third-party applications with exchange data from a unified data model. <p>The Exchange DataFeed Data Model API provides mappings for enumerations used in our Real-Time DataFeed products and should be used in conjunction with the DataFeed Data Model documentation available for each product. <p>The initial beta version of this API is limited to include mapping tables for the product codes only.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 0.20.13
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.ExchangeDataFeedDataModel==0.20.13
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.ExchangeDataFeedDataModel==0.20.13
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.ExchangeDataFeedDataModel
from fds.sdk.ExchangeDataFeedDataModel.api import fact_set_product_codes_api
from fds.sdk.ExchangeDataFeedDataModel.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.ExchangeDataFeedDataModel.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = fact_set_product_codes_api.FactSetProductCodesApi(api_client)
    format = "json" # str | The format of the output file.<p>**Try it Out** - All formats available</p> (optional) if omitted the server will use the default value of "json"

    try:
        # Request the enumeration table for FactSet product codes.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_products(format=format)

        pprint(api_response)
    except fds.sdk.ExchangeDataFeedDataModel.ApiException as e:
        print("Exception when calling FactSetProductCodesApi->get_products: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Request the enumeration table for FactSet product codes.
    #     api_response, http_status_code, response_headers = api_instance.get_products_with_http_info(format=format)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ExchangeDataFeedDataModel.ApiException as e:
    #     print("Exception when calling FactSetProductCodesApi->get_products: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Request the enumeration table for FactSet product codes.
    #     async_result = api_instance.get_products_async(format=format)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.ExchangeDataFeedDataModel.ApiException as e:
    #     print("Exception when calling FactSetProductCodesApi->get_products: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Request the enumeration table for FactSet product codes.
    #     async_result = api_instance.get_products_with_http_info_async(format=format)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ExchangeDataFeedDataModel.ApiException as e:
    #     print("Exception when calling FactSetProductCodesApi->get_products: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.ExchangeDataFeedDataModel

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.ExchangeDataFeedDataModel

configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.ExchangeDataFeedDataModel

configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.ExchangeDataFeedDataModel

configuration = fds.sdk.ExchangeDataFeedDataModel.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/RTDataModel*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*FactSetProductCodesApi* | [**get_products**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedDataModel/v1/docs/FactSetProductCodesApi.md#get_products) | **GET** /products | Request the enumeration table for FactSet product codes.


## Documentation For Models

 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedDataModel/v1/docs/ErrorResponse.md)
 - [RTDataModelProducts](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedDataModel/v1/docs/RTDataModelProducts.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.ExchangeDataFeedDataModel.apis and fds.sdk.ExchangeDataFeedDataModel.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.ExchangeDataFeedDataModel.api.default_api import DefaultApi`
- `from fds.sdk.ExchangeDataFeedDataModel.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.ExchangeDataFeedDataModel
from fds.sdk.ExchangeDataFeedDataModel.apis import *
from fds.sdk.ExchangeDataFeedDataModel.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

